
#include ".\sgui\sgui.bas"
#include ".\sgui\checkmarkgadget.bas"
#include ".\sgui\simplegadget.bas"
#include ".\sgui\menu.bas"
#include ".\sgui\messagebox.bas"

#undef black
#undef white

#include ".\game.grid.bas"
#include ".\game.graphics.bas"
#include ".\game.colors.bas"
#include ".\game.language.bas"
#include ".\game.engine.bas"

using sGUI

declare sub Display(as string)
declare function EmptyCell(as TFourInARowGrid, as integer) as integer
declare sub InsertAndCheckGameState(as integer, as integer)
declare sub Restart()

dim as boolean ComputerPlaysWhite = false, ComputerPlaysBlack = true
dim vs as TVelenaStrength = vsWeak

#define MENU_HEIGHT 20
#define LABEL_HEIGHT 24

const SW = 8 * 48, SH = MENU_HEIGHT + 8 * 48 + LABEL_HEIGHT
screenres SW, SH, 32
width SW \ 8, SH \ 16
windowtitle sWindowTitle
InitGFX

dim shared graph as TFourInARowGraphics
graph.Initialize(48, 0, MENU_HEIGHT, gridColor, "empty.png", "white.png", "black.png")

dim shared as EventHandle ptr event
event = new EventHandle

dim as integer item_help, item_quit, item_about, item_move, item_auto, item_new, item_weak, _
item_normal, item_strong

Menu(event, 1, 0, 1, sGame)
item_quit = Menu(event, 1, 1, 1, sQuit)
Menu(event, 1, 2, 4, "")
item_about = Menu(event, 1, 3, 1, sAbout)
Menu(event, 2, 0, 1, sMoves)
item_move = Menu(event, 2, 1, 1, sForceMove)
item_auto = Menu(event, 2, 2, 3, sAuto)
Menu(event, 2, 3, 4, "")
item_weak = Menu(event, 2, 4, iif(vs = vsWeak, 3, 1), sWeak)
item_normal = Menu(event, 2, 5, iif(vs = vsNormal, 3, 1), sNormal)
item_strong = Menu(event, 2, 6, iif(vs = vsStrong, 3, 1), sStrong)
Menu(event, 3, 0, 1, sBoard)
item_new = Menu(event, 3, 1, 1, sNew)
MenuOn(event)

dim as Gadget ptr b1, b2, b3, b4, b5, b6, b7
b1 = AddSimpleGadget(event, 0 * 48 + 4 + 24, MENU_HEIGHT + 7 * 48 + 12, 40, 24, "")
b2 = AddSimpleGadget(event, 1 * 48 + 4 + 24, MENU_HEIGHT + 7 * 48 + 12, 40, 24, "")
b3 = AddSimpleGadget(event, 2 * 48 + 4 + 24, MENU_HEIGHT + 7 * 48 + 12, 40, 24, "")
b4 = AddSimpleGadget(event, 3 * 48 + 4 + 24, MENU_HEIGHT + 7 * 48 + 12, 40, 24, "")
b5 = AddSimpleGadget(event, 4 * 48 + 4 + 24, MENU_HEIGHT + 7 * 48 + 12, 40, 24, "")
b6 = AddSimpleGadget(event, 5 * 48 + 4 + 24, MENU_HEIGHT + 7 * 48 + 12, 40, 24, "")
b7 = AddSimpleGadget(event, 6 * 48 + 4 + 24, MENU_HEIGHT + 7 * 48 + 12, 40, 24, "")
GadgetOn(b1, b7)

dim shared grid as TFourInARowGrid
dim shared piece as TPiece
dim as integer x, y
dim shared history as string
dim shared gameOver as boolean
dim question as string

#macro ForceVelenaMove
if (cbool(piece = pwhite) and not ComputerPlaysWhite) _
or (cbool(piece = pblack) and not ComputerPlaysBlack) then
  ComputerPlaysWhite = not ComputerPlaysWhite
  ComputerPlaysBlack = not ComputerPlaysBlack
end if
#endmacro

Restart()

do
  event->xSleep(1)
  if event->GADGETMESSAGE then
    select case event->GADGETMESSAGE
      case b1 : x = 1
      case b2 : x = 2
      case b3 : x = 3
      case b4 : x = 4
      case b5 : x = 5
      case b6 : x = 6
      case b7 : x = 7
    end select
    if not gameOver then
      y = EmptyCell(grid, x)
      if (x > 0) andalso (y > 0) then
        InsertAndCheckGameState(x, y)
      end if
    end if
  end if
  
  if event->MENUMESSAGE then
    select case event->MENUMESSAGE
      case item_quit
        event->EXITEVENT = 1
      case item_about
        MessageBox(10, MENU_HEIGHT + 10, sAboutText, sWindowTitle, MBType_OK)
      case item_move
        ForceVelenaMove
      case item_auto
        if GetChecked(event, 2, 2) then
          Menu(event, 2, 2, 1, sAuto)
          ComputerPlaysWhite = false
          ComputerPlaysBlack = false
        else
          Menu(event, 2, 2, 3, sAuto)
          ComputerPlaysWhite = piece = pblack
          ComputerPlaysBlack = piece = pwhite
        end if
      case item_new
        Restart()
      case item_weak, item_normal, item_strong
        select case event->MENUMESSAGE
          case item_weak
            vs = vsWeak
          case item_normal
            vs = vsNormal
          case item_strong
            vs = vsStrong
        end select
        Menu(event, 2, 4, iif(vs = vsWeak, 3, 1), sWeak)
        Menu(event, 2, 5, iif(vs = vsNormal, 3, 1), sNormal)
        Menu(event, 2, 6, iif(vs = vsStrong, 3, 1), sStrong)
    end select
  end if
  
  if KEY <> "" then
    select case KEY
      case "m"
        ForceVelenaMove
      case chr(27)
        event->EXITEVENT = 1
    end select
  end if
  
  if not gameOver then
    if (cbool(piece = pwhite) and ComputerPlaysWhite) or (cbool(piece = pblack) and ComputerPlaysBlack) then
      
      question = VSSymbol(vs) & history & "0"
      x = VelenaAnswer(question)
      
      if (x >= 1) and (x <= 7) then
        y = EmptyCell(grid, x)
        InsertAndCheckGameState(x, y)
      else
        select case x
          case ANSWER_SYNTAX_ERROR
            '
          case ANSWER_POSITIONAL_ERROR
            '
        end select
        ComputerPlaysWhite = false
        ComputerPlaysBlack = false
      end if
    end if
  end if
loop until event->EXITEVENT

graph.FreeImages
delete event
end

sub Display(s as string)
  'FillA(0, MENU_HEIGHT + 8 * 48, 8 * 48, LABEL_HEIGHT, windowColor, 1)
  line (0, MENU_HEIGHT + 8 * 48)-(8 * 48, MENU_HEIGHT + 8 * 48 + LABEL_HEIGHT), sGUI.backgroundColor, BF
  draw string(0 + 4, MENU_HEIGHT + 8 * 48 + 4), s, whitePieceColor
end sub

function EmptyCell(aGrid as TFourInARowGrid, aCol as integer) as integer
  dim as integer result = 0
  dim as integer y = 0
  while (y < 6) and (result = 0)
    y += 1
    if aGrid.array(aCol, y) = pnothing then result = y
  wend
  function = result
end function

sub InsertAndCheckGameState(aX as integer, aY as integer)
  dim as integer whitePoints, blackPoints
  dim s as string
  graph.InsertPiece(aX, aY, iif(piece=pwhite, clWhite, clBlack))
  grid.Insert(piece, aX)
  piece *= pblack
  history += str(aX)
  whitePoints = grid.GridPoints(pwhite)
  blackPoints = grid.GridPoints(pblack)
  if cbool(whitePoints >= 4) or cbool(blackPoints >= 4) or grid.IsFull then
    gameOver = true
    if whitePoints >= 4 then
      s = sWhiteWins
    elseif blackPoints >= 4 then
      s = sBlackWins
    elseif grid.IsFull then
      s = sDraw
    end if
  else
    s = iif(piece = pwhite, sWhiteMove, sBlackMove)
  end if
  Display(s)
end sub

sub Restart()
  piece = pwhite
  history = ""
  grid.Empty
  graph.DrawGrid
  gameOver = false
  Display(iif(piece = pwhite, sWhiteMove, sBlackMove))
end sub
