
#include "game.grid.definitions.bas"
#include "game.grid.constants.bas"
#include "game.grid.functions.bas"
#include "game.grid.strings.bas"
#include "game.grid.lines.bas"

type TFourInARowGrid
  as TPiece array(one - 1 to seven + 1, one - 1 to six + 1)
  declare sub Empty
  declare sub Copy(p() as TPiece)
  declare function IsFull as boolean
  declare function IsEmpty as boolean
  declare function Insert(p as TPiece, x as integer) as boolean
  declare function Fill(s as string) as boolean
  declare function LinePoints(l as _line, p as TPiece) as integer
  declare function GridPoints(p as TPiece) as integer
  declare function AsString as string
end type

sub TFourInARowGrid.Empty
  dim as integer x, y
  for x = one - 1 to seven + 1
    for y = one - 1 to six + 1
      array(x, y) = pborder
    next y
  next x
  for x = one to seven
    for y = one to six
      array(x, y) = pnothing
    next y
  next x
end sub

sub TFourInARowGrid.Copy(p() as TPiece)
  for x as integer = one to seven
    for y as integer = one to six
      array(x, y) = p(x, y)
    next y
  next x
end sub

function TFourInARowGrid.IsFull as boolean
  dim result as boolean = true
  dim x as integer = one
  do while cbool(x <= seven) and result
    if array(x, six) = pnothing then result = false
    x += 1
  loop
  return result
end function

function TFourInARowGrid.IsEmpty as boolean
  dim result as boolean = true
  dim x as integer = one
  do while cbool(x <= seven) and result
    if array(x, six) <> pnothing then result = false
    x += 1
  loop
  return result
end function

function TFourInARowGrid.Insert(p as TPiece, x as integer) as boolean
  dim y as integer = six
  dim result as boolean = (array(x, y) = pnothing)
  if result then
    do while (array(x, y - 1) = pnothing)
      y -= 1
    loop
    array(x, y) = p
  end if
  return result
end function

function TFourInARowGrid.Fill(s as string) as boolean
  dim result as boolean = IsEmpty and IsValidFormat(s) andalso IsValidSequence(s)
  dim p as TPiece = pwhite
  if result then
    for i as integer = 0 to len(s) - 1
      Insert(p, AscToInt(s[i]))
      p *= pblack
    next i
  end if
  return result
end function

function TFourInARowGrid.LinePoints(l as _line, p as TPiece) as integer
  dim as integer x, y, n=0, o=0, nmx=0, omx=0, result
  x = l.x1
  y = l.y1
  do while array(x, y) <> pborder
    if array(x, y) = p then n += 1 else n = 0
    if array(x, y) <> pblack * p then o += 1 else o = 0
    if n > nmx then nmx = n
    if o > omx then omx = o
    x += l.dx
    y += l.dy 
  loop
  if omx < 4 then result = 0 else result = nmx
  return result
end function

function TFourInARowGrid.GridPoints(p as TPiece) as integer
  dim as integer i, j, k = 0
  for i = 0 to 24
    j = LinePoints(lines(i), p)
	if j > k then k = j
  next i
  return k
end function

function TFourInARowGrid.AsString as string
  dim result as string = ""
  for y as integer = six + 1 to one - 1 step -1
    for x as integer = one - 1 to seven + 1
      select case array(x, y)
        case pnothing
          result = result & " "
        case pwhite
          result = result & "X"
        case pblack
          result = result & "O"
        case pborder
          result = result & "#"
      end select
    next x
    result = result & chr(13, 10)
  next y
  return result
end function



